@@ -149,8 +149,8 @@ template<class T, class Integral> inline Color4<T> fromSrgbAlphaIntegral(const V
 
 /* RGB -> sRGB conversion */
 template<class T> Vector3<typename Color3<T>::FloatingPointType> toSrgb(typename std::enable_if<std::is_floating_point<T>::value, const Color3<T>&>::type rgb) {
-    constexpr const T a(0.055);
-    return lerp(rgb*T(12.92), (T(1.0) + a)*pow(rgb, T(1.0)/T(2.4)) - Vector3<T>{a}, rgb > Vector3<T>(0.0031308));
+    constexpr const T a = T(0.055);
+    return lerp(rgb*T(12.92), (T(1.0) + a)*pow(rgb, T(1.0)/T(2.4)) - Vector3<T>{a}, rgb > Vector3<T>(T(0.0031308)));
 }
 template<class T> Vector4<typename Color4<T>::FloatingPointType> toSrgbAlpha(typename std::enable_if<std::is_floating_point<T>::value, const Color4<T>&>::type rgba) {
     return {toSrgb<T>(rgba.rgb()), rgba.a()};
@@ -1179,7 +1179,7 @@ inline Color4<Float> operator "" _srgbaf(unsigned long long value) {
 }
 
 /**
-@debugoperator{Magnum::Math::Color3}
+@debugoperator{Color3}
 
 Prints the value as hex color (e.g. @cb{.shell-session} #ff33aa @ce). Other
 underlying types are handled by @ref operator<<(Corrade::Utility::Debug&, const Vector<size, T>&).
@@ -1187,7 +1187,7 @@ underlying types are handled by @ref operator<<(Corrade::Utility::Debug&, const
 MAGNUM_EXPORT Corrade::Utility::Debug& operator<<(Corrade::Utility::Debug& debug, const Color3<UnsignedByte>& value);
 
 /**
-@debugoperator{Magnum::Math::Color4}
+@debugoperator{Color4}
 
 Prints the value as hex color (e.g. @cb{.shell-session} #9933aaff @ce). Other
 underlying types are handled by @ref operator<<(Corrade::Utility::Debug&, const Vector<size, T>&).
