@@ -39,6 +39,10 @@
 #include "Magnum/Math/Vector2.h"
 #include "Magnum/Platform/Platform.h"
 
+#ifdef MAGNUM_TARGET_GL
+#include "Magnum/GL/GL.h"
+#endif
+
 #ifdef CORRADE_TARGET_WINDOWS /* Windows version of SDL2 redefines main(), we don't want that */
 #define SDL_MAIN_HANDLED
 #endif
@@ -362,6 +366,9 @@ class Sdl2Application {
         };
 
         class Configuration;
+        #ifdef MAGNUM_TARGET_GL
+        class GLConfiguration;
+        #endif
         class InputEvent;
         class KeyEvent;
         class MouseEvent;
@@ -371,30 +378,55 @@ class Sdl2Application {
         class TextInputEvent;
         class TextEditingEvent;
 
+        #ifdef MAGNUM_TARGET_GL
         /**
-         * @brief Default constructor
-         * @param arguments     Application arguments
-         * @param configuration Configuration
+         * @brief Construct with given configuration for OpenGL context
+         * @param arguments         Application arguments
+         * @param configuration     Application configuration
+         * @param glConfiguration   OpenGL context configuration
          *
          * Creates application with default or user-specified configuration.
          * See @ref Configuration for more information. The program exits if
-         * the context cannot be created, see @ref tryCreateContext() for an
+         * the context cannot be created, see @ref tryCreate() for an
          * alternative.
+         *
+         * @note This function is available only if Magnum is compiled with
+         *      @ref MAGNUM_TARGET_GL enabled (done by default). See
+         *      @ref building-features for more information.
          */
-        #ifdef DOXYGEN_GENERATING_OUTPUT
-        explicit Sdl2Application(const Arguments& arguments, const Configuration& configuration = Configuration());
-        #else
-        /* To avoid "invalid use of incomplete type" */
-        explicit Sdl2Application(const Arguments& arguments, const Configuration& configuration);
-        explicit Sdl2Application(const Arguments& arguments);
+        explicit Sdl2Application(const Arguments& arguments, const Configuration& configuration, const GLConfiguration& glConfiguration);
         #endif
 
         /**
-         * @brief Constructor
+         * @brief Construct with given configuration
+         *
+         * If @ref Configuration::WindowFlag::Contextless is present or Magnum
+         * was not built with @ref MAGNUM_TARGET_GL, this creates a window
+         * without any GPU context attached, leaving that part on the user.
+         *
+         * If none of the flags is present and Magnum was built with
+         * @ref MAGNUM_TARGET_GL, this is equivalent to calling
+         * @ref Sdl2Application(const Arguments&, const Configuration&, const GLConfiguration&)
+         * with default-constructed @ref GLConfiguration.
+         *
+         * See also @ref building-features for more information.
+         */
+        explicit Sdl2Application(const Arguments& arguments, const Configuration& configuration);
+
+        /**
+         * @brief Construct with default configuration
+         *
+         * Equivalent to calling @ref Sdl2Application(const Arguments&, const Configuration&)
+         * with default-constructed @ref Configuration.
+         */
+        explicit Sdl2Application(const Arguments& arguments);
+
+        /**
+         * @brief Construct without creating a window
          * @param arguments     Application arguments
          *
-         * Unlike above, the context is not created and must be created later
-         * with @ref createContext() or @ref tryCreateContext().
+         * Unlike above, the window is not created and must be created later
+         * with @ref create() or @ref tryCreate().
          */
         explicit Sdl2Application(const Arguments& arguments, NoCreateT);
 
@@ -460,34 +492,100 @@ class Sdl2Application {
            faster than public pure virtual destructor */
         ~Sdl2Application();
 
+        #ifdef MAGNUM_TARGET_GL
         /**
-         * @brief Create context with given configuration
+         * @brief Create a window with given configuration for OpenGL context
+         * @param configuration     Application configuration
+         * @param glConfiguration   OpenGL context configuration
          *
-         * Must be called if and only if the context wasn't created by the
-         * constructor itself. Error message is printed and the program exits
-         * if the context cannot be created, see @ref tryCreateContext() for an
-         * alternative.
+         * Must be called only if the context wasn't created by the constructor
+         * itself, i.e. when passing @ref NoCreate to it. Error message is
+         * printed and the program exits if the context cannot be created, see
+         * @ref tryCreate() for an alternative.
          *
-         * On desktop GL, if version is not specified in @p configuration, the
-         * application first tries to create core context (OpenGL 3.2+ on
+         * On desktop GL, if version is not specified in @p glConfiguration,
+         * the application first tries to create core context (OpenGL 3.2+ on
          * macOS, OpenGL 3.1+ elsewhere) and if that fails, falls back to
          * compatibility OpenGL 2.1 context.
+         *
+         * @note This function is available only if Magnum is compiled with
+         *      @ref MAGNUM_TARGET_GL enabled (done by default). See
+         *      @ref building-features for more information.
          */
-        #ifdef DOXYGEN_GENERATING_OUTPUT
-        void createContext(const Configuration& configuration = Configuration());
-        #else
-        /* To avoid "invalid use of incomplete type" */
-        void createContext(const Configuration& configuration);
-        void createContext();
+        void create(const Configuration& configuration, const GLConfiguration& glConfiguration);
+        #endif
+
+        /**
+         * @brief Create a window with given configuration
+         *
+         * If @ref Configuration::WindowFlag::Contextless is present or Magnum
+         * was not built with @ref MAGNUM_TARGET_GL, this creates a window
+         * without any GPU context attached, leaving that part on the user.
+         *
+         * If none of the flags is present and Magnum was built with
+         * @ref MAGNUM_TARGET_GL, this is equivalent to calling
+         * @ref create(const Configuration&, const GLConfiguration&) with
+         * default-constructed @ref GLConfiguration.
+         *
+         * See also @ref building-features for more information.
+         */
+        void create(const Configuration& configuration);
+
+        /**
+         * @brief Create a window with default configuration and OpenGL context
+         *
+         * Equivalent to calling @ref create(const Configuration&) with
+         * default-constructed @ref Configuration.
+         */
+        void create();
+
+        #if defined(MAGNUM_BUILD_DEPRECATED) && defined(MAGNUM_TARGET_GL)
+        /** @brief @copybrief create(const Configuration&, const GLConfiguration&)
+         * @deprecated Use @ref create(const Configuration&, const GLConfiguration&) instead.
+         */
+        CORRADE_DEPRECATED("use create(const Configuration&, const GLConfiguration&) instead") void createContext(const Configuration& configuration) {
+            create(configuration);
+        }
+
+        /** @brief @copybrief create()
+         * @deprecated Use @ref create() instead.
+         */
+        CORRADE_DEPRECATED("use create() instead") void createContext() {
+            create();
+        }
+        #endif
+
+        #ifdef MAGNUM_TARGET_GL
+        /**
+         * @brief Try to create context with given configuration for OpenGL context
+         *
+         * Unlike @ref create(const Configuration&, const GLConfiguration&)
+         * returns @cpp false @ce if the context cannot be created,
+         * @cpp true @ce otherwise.
+         *
+         * @note This function is available only if Magnum is compiled with
+         *      @ref MAGNUM_TARGET_GL enabled (done by default). See
+         *      @ref building-features for more information.
+         */
+        bool tryCreate(const Configuration& configuration, const GLConfiguration& glConfiguration);
         #endif
 
         /**
          * @brief Try to create context with given configuration
          *
-         * Unlike @ref createContext() returns @cpp false @ce if the context
-         * cannot be created, @cpp true @ce otherwise.
+         * Unlike @ref create(const Configuration&) returns @cpp false @ce if
+         * the context cannot be created, @cpp true @ce otherwise.
          */
-        bool tryCreateContext(const Configuration& configuration);
+        bool tryCreate(const Configuration& configuration);
+
+        #if defined(MAGNUM_BUILD_DEPRECATED) && defined(MAGNUM_TARGET_GL)
+        /** @brief @copybrief tryCreate(const Configuration&, const GLConfiguration&)
+         * @deprecated Use @ref tryCreate(const Configuration&, const GLConfiguration&) instead.
+         */
+        CORRADE_DEPRECATED("use tryCreate(const Configuration&, const GLConfiguration&) instead") bool tryCreateContext(const Configuration& configuration) {
+            return tryCreate(configuration);
+        }
+        #endif
 
         /** @{ @name Screen handling */
 
@@ -568,16 +666,16 @@ class Sdl2Application {
          * @brief Viewport event
          *
          * Called when window size changes. The default implementation does
-         * nothing, if you want to respond to size changes, you should pass the
-         * new size to @ref DefaultFramebuffer::setViewport() and possibly
-         * elsewhere (to @ref SceneGraph::Camera::setViewport(), other
-         * framebuffers...).
+         * nothing. If you want to respond to size changes, you should pass the
+         * new size to @ref GL::DefaultFramebuffer::setViewport() (if using
+         * OpenGL) and possibly elsewhere (to
+         * @ref SceneGraph::Camera::setViewport(), other framebuffers...).
          *
          * Note that this function might not get called at all if the window
          * size doesn't change. You should configure the initial state of your
          * cameras, framebuffers etc. in application constructor rather than
          * relying on this function to be called. Viewport of default
-         * framebuffer can be retrieved via @ref DefaultFramebuffer::viewport().
+         * framebuffer can be retrieved via @ref GL::DefaultFramebuffer::viewport().
          */
         virtual void viewportEvent(const Vector2i& size);
 
@@ -585,9 +683,10 @@ class Sdl2Application {
          * @brief Draw event
          *
          * Called when the screen is redrawn. You should clean the framebuffer
-         * using @ref DefaultFramebuffer::clear() and then add your own drawing
-         * functions. After drawing is finished, call @ref swapBuffers(). If
-         * you want to draw immediately again, call also @ref redraw().
+         * using @ref GL::DefaultFramebuffer::clear() (if using OpenGL) and
+         * then add your own drawing functions. After drawing is finished, call
+         * @ref swapBuffers(). If you want to draw immediately again, call also
+         * @ref redraw().
          */
         virtual void drawEvent() = 0;
 
@@ -759,25 +858,35 @@ class Sdl2Application {
 
         #ifndef CORRADE_TARGET_EMSCRIPTEN
         SDL_Window* _window;
-        SDL_GLContext _glContext;
         UnsignedInt _minimalLoopPeriod;
+        #endif
+
+        #ifdef MAGNUM_TARGET_GL
+        #ifndef CORRADE_TARGET_EMSCRIPTEN
+        SDL_GLContext _glContext;
         #else
         SDL_Surface* _glContext;
         #endif
-
-        std::unique_ptr<Platform::Context> _context;
+        std::unique_ptr<Platform::GLContext> _context;
+        #endif
 
         Flags _flags;
 };
 
+#ifdef MAGNUM_TARGET_GL
 /**
-@brief Configuration
+@brief OpenGL context configuration
 
 The created window is always with double-buffered OpenGL context and 24bit
 depth buffer.
-@see @ref Sdl2Application(), @ref createContext(), @ref tryCreateContext()
+
+@note This function is available only if Magnum is compiled with
+    @ref MAGNUM_TARGET_GL enabled (done by default). See @ref building-features
+    for more information.
+
+@see @ref Sdl2Application(), @ref create(), @ref tryCreate()
 */
-class Sdl2Application::Configuration {
+class Sdl2Application::GLConfiguration {
     public:
         #ifndef CORRADE_TARGET_EMSCRIPTEN
         /**
@@ -811,6 +920,129 @@ class Sdl2Application::Configuration {
         #endif
         #endif
 
+        explicit GLConfiguration();
+        ~GLConfiguration();
+
+        #ifndef CORRADE_TARGET_EMSCRIPTEN
+        /**
+         * @brief Context flags
+         *
+         * @note Not available in @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten".
+         */
+        Flags flags() const { return _flags; }
+
+        /**
+         * @brief Set context flags
+         * @return Reference to self (for method chaining)
+         *
+         * Default is no flag. See also @ref GL::Context::flags().
+         * @note Not available in @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten".
+         */
+        GLConfiguration& setFlags(Flags flags) {
+            _flags = flags;
+            return *this;
+        }
+
+        /**
+         * @brief Context version
+         *
+         * @note Not available in @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten".
+         */
+        GL::Version version() const { return _version; }
+        #endif
+
+        /**
+         * @brief Set context version
+         *
+         * If requesting version greater or equal to OpenGL 3.1, core profile
+         * is used. The created context will then have any version which is
+         * backwards-compatible with requested one. Default is
+         * @ref GL::Version::None, i.e. any provided version is used.
+         * @note In @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten" this function
+         *      does nothing (@ref GL::Version::GLES200 or
+         *      @ref GL::Version::GLES300 is used implicitly based on the
+         *      target).
+         */
+        GLConfiguration& setVersion(GL::Version version) {
+            #ifndef CORRADE_TARGET_EMSCRIPTEN
+            _version = version;
+            #else
+            static_cast<void>(version);
+            #endif
+            return *this;
+        }
+
+        /** @brief Sample count */
+        Int sampleCount() const { return _sampleCount; }
+
+        /**
+         * @brief Set sample count
+         * @return Reference to self (for method chaining)
+         *
+         * Default is @cpp 0 @ce, thus no multisampling. See also
+         * @ref GL::Renderer::Feature::Multisampling.
+         */
+        GLConfiguration& setSampleCount(Int count) {
+            _sampleCount = count;
+            return *this;
+        }
+
+        #ifndef CORRADE_TARGET_EMSCRIPTEN
+        /**
+         * @brief sRGB-capable default framebuffer
+         *
+         * @note Not available in @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten".
+         */
+        bool isSRGBCapable() const { return _sRGBCapable; }
+
+        /**
+         * @brief Set sRGB-capable default framebuffer
+         * @return Reference to self (for method chaining)
+         *
+         * Default is @cpp false @ce. See also
+         * @ref GL::Renderer::Feature::FramebufferSRGB.
+         * @note Not available in @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten".
+         */
+        GLConfiguration& setSRGBCapable(bool enabled) {
+            _sRGBCapable = enabled;
+            return *this;
+        }
+        #endif
+
+    private:
+        Int _sampleCount;
+        #ifndef CORRADE_TARGET_EMSCRIPTEN
+        GL::Version _version;
+        Flags _flags;
+        bool _sRGBCapable;
+        #endif
+};
+
+#ifndef CORRADE_TARGET_EMSCRIPTEN
+CORRADE_ENUMSET_OPERATORS(Sdl2Application::GLConfiguration::Flags)
+#endif
+#endif
+
+/**
+@brief Configuration
+
+@see @ref Sdl2Application(), @ref GLConfiguration, @ref create(),
+    @ref tryCreate()
+*/
+class Sdl2Application::Configuration {
+    public:
+        #if defined(MAGNUM_BUILD_DEPRECATED) && defined(MAGNUM_TARGET_GL) && !defined(CORRADE_TARGET_EMSCRIPTEN)
+        /** @brief @copybrief GLConfiguration::Flag
+         * @deprecated Use @ref GLConfiguration::Flag instead.
+         */
+        typedef GLConfiguration::Flag Flag;
+
+        /** @brief @copybrief GLConfiguration::Flags
+         * @deprecated Use @ref GLConfiguration::Flags instead.
+         */
+        typedef GLConfiguration::Flags Flags;
+        #endif
+
         /**
          * @brief Window flag
          *
@@ -842,7 +1074,16 @@ class Sdl2Application::Configuration {
             Hidden = SDL_WINDOW_HIDDEN,             /**< Hidden window */
             Maximized = SDL_WINDOW_MAXIMIZED,       /**< Maximized window */
             Minimized = SDL_WINDOW_MINIMIZED,       /**< Minimized window */
-            MouseLocked = SDL_WINDOW_INPUT_GRABBED  /**< Window with mouse locked */
+            MouseLocked = SDL_WINDOW_INPUT_GRABBED, /**< Window with mouse locked */
+
+            /**
+             * Do not create any GPU context. Use together with
+             * @ref Sdl2Application(const Arguments&, const Configuration&),
+             * @ref create(const Configuration&) or
+             * @ref tryCreate(const Configuration&) to prevent implicit
+             * creation of an OpenGL context.
+             */
+            Contextless = 1u << 31 /* Hope this won't ever conflict with anything */
         };
 
         /**
@@ -922,46 +1163,31 @@ class Sdl2Application::Configuration {
             return *this;
         }
 
+        #if defined(MAGNUM_BUILD_DEPRECATED) && defined(MAGNUM_TARGET_GL)
         #ifndef CORRADE_TARGET_EMSCRIPTEN
-        /**
-         * @brief Context flags
-         *
-         * @note Not available in @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten".
+        /** @brief @copybrief GLConfiguration::flags()
+         * @deprecated Use @ref GLConfiguration::flags() instead.
          */
-        Flags flags() const { return _flags; }
+        CORRADE_DEPRECATED("use GLConfiguration::flags() instead") GLConfiguration::Flags flags() const { return _flags; }
 
-        /**
-         * @brief Set context flags
-         * @return Reference to self (for method chaining)
-         *
-         * Default is no flag. See also @ref Context::flags().
-         * @note Not available in @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten".
+        /** @brief @copybrief GLConfiguration::setFlags()
+         * @deprecated Use @ref GLConfiguration::setFlags() instead.
          */
-        Configuration& setFlags(Flags flags) {
+        CORRADE_DEPRECATED("use GLConfiguration::setFlags() instead") Configuration& setFlags(GLConfiguration::Flags flags) {
             _flags = flags;
             return *this;
         }
 
-        /**
-         * @brief Context version
-         *
-         * @note Not available in @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten".
+        /** @brief @copybrief GLConfiguration::version()
+         * @deprecated Use @ref GLConfiguration::version() instead.
          */
-        Version version() const { return _version; }
+        CORRADE_DEPRECATED("use GLConfiguration::version() instead") GL::Version version() const { return _version; }
         #endif
 
-        /**
-         * @brief Set context version
-         *
-         * If requesting version greater or equal to OpenGL 3.1, core profile
-         * is used. The created context will then have any version which is
-         * backwards-compatible with requested one. Default is
-         * @ref Version::None, i.e. any provided version is used.
-         * @note In @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten" this function
-         *      does nothing (@ref Version::GLES200 or @ref Version::GLES300 is
-         *      used implicitly based on the target).
+        /** @brief @copybrief GLConfiguration::setVersion()
+         * @deprecated Use @ref GLConfiguration::setVersion() instead.
          */
-        Configuration& setVersion(Version version) {
+        CORRADE_DEPRECATED("use GLConfiguration::setVersion() instead") Configuration& setVersion(GL::Version version) {
             #ifndef CORRADE_TARGET_EMSCRIPTEN
             _version = version;
             #else
@@ -970,41 +1196,34 @@ class Sdl2Application::Configuration {
             return *this;
         }
 
-        /** @brief Sample count */
-        Int sampleCount() const { return _sampleCount; }
-
-        /**
-         * @brief Set sample count
-         * @return Reference to self (for method chaining)
-         *
-         * Default is @cpp 0 @ce, thus no multisampling. See also
-         * @ref Renderer::Feature::Multisampling.
+        /** @brief @copybrief GLConfiguration::sampleCount()
+         * @deprecated Use @ref GLConfiguration::sampleCount() instead.
          */
-        Configuration& setSampleCount(Int count) {
+        CORRADE_DEPRECATED("use GLConfiguration::sampleCount() instead") Int sampleCount() const { return _sampleCount; }
+
+        /** @brief @copybrief GLConfiguration::setSampleCount()
+         * @deprecated Use @ref GLConfiguration::setSampleCount() instead.
+         */
+        CORRADE_DEPRECATED("use GLConfiguration::setSampleCount() instead") Configuration& setSampleCount(Int count) {
             _sampleCount = count;
             return *this;
         }
 
         #ifndef CORRADE_TARGET_EMSCRIPTEN
-        /**
-         * @brief sRGB-capable default framebuffer
-         *
-         * @note Not available in @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten".
+        /** @brief @copybrief GLConfiguration::isSRGBCapable()
+         * @deprecated Use @ref GLConfiguration::isSRGBCapable() instead.
          */
-        bool isSRGBCapable() const { return _sRGBCapable; }
+        CORRADE_DEPRECATED("use GLConfiguration::isSRGBCapable() instead") bool isSRGBCapable() const { return _sRGBCapable; }
 
-        /**
-         * @brief Set sRGB-capable default framebuffer
-         * @return Reference to self (for method chaining)
-         *
-         * Default is @cpp false @ce. See also @ref Renderer::Feature::FramebufferSRGB.
-         * @note Not available in @ref CORRADE_TARGET_EMSCRIPTEN "Emscripten".
+        /** @brief @copybrief GLConfiguration::setSRGBCapable()
+         * @deprecated Use @ref GLConfiguration::setSRGBCapable() instead.
          */
-        Configuration& setSRGBCapable(bool enabled) {
+        CORRADE_DEPRECATED("use GLConfiguration::setSRGBCapable() instead") Configuration& setSRGBCapable(bool enabled) {
             _sRGBCapable = enabled;
             return *this;
         }
         #endif
+        #endif
 
     private:
         #if !defined(CORRADE_TARGET_EMSCRIPTEN) && !defined(CORRADE_TARGET_IOS)
@@ -1012,12 +1231,14 @@ class Sdl2Application::Configuration {
         #endif
         Vector2i _size;
         WindowFlags _windowFlags;
+        #if defined(MAGNUM_BUILD_DEPRECATED) && defined(MAGNUM_TARGET_GL)
         Int _sampleCount;
         #ifndef CORRADE_TARGET_EMSCRIPTEN
-        Version _version;
+        GL::Version _version;
         Flags _flags;
         bool _sRGBCapable;
         #endif
+        #endif
 };
 
 /**
@@ -1714,9 +1935,6 @@ typedef BasicScreenedApplication<Sdl2Application> ScreenedApplication;
 #endif
 
 CORRADE_ENUMSET_OPERATORS(Sdl2Application::Flags)
-#ifndef CORRADE_TARGET_EMSCRIPTEN
-CORRADE_ENUMSET_OPERATORS(Sdl2Application::Configuration::Flags)
-#endif
 CORRADE_ENUMSET_OPERATORS(Sdl2Application::Configuration::WindowFlags)
 CORRADE_ENUMSET_OPERATORS(Sdl2Application::InputEvent::Modifiers)
 CORRADE_ENUMSET_OPERATORS(Sdl2Application::MouseMoveEvent::Buttons)
